#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-
"""
Name : conkyWeather_arclance v6.0
Author: arclance
Date_Created:2011-10-29
Date_Updated:2012-08-13
	
Reads current weather information from the Wunderground API and formats it for display parsing by a lua script.

ChangeLog
v6.0 - 2012-08-13
	Initial Release
	
v6.X - pre release
	2012-07-22
	fiveDay weather code completely converted to lua, remove all code that is now in the lua script.
	improve stability of download code (add catcher for httplib.BadStatusLine to tryconnect)
	drop support for all weather image sources except VClouds
		old python functions for other image sources could be converted to lua by using the VClouds code as an example
	
	2012-07-23
	added hourly weather output to lua script using buttons
		converted last of the five day code to lua (image selection, moon phase calculation)

	2012-07-25
	remove scrolling alerts code, now have working lua function to replace ${scroll} from conky

	2012-07-28
	change settings for sattelite image thanks to guide from API
	
	2012-07-29
	get history data for today (has better rain/snow amount data)
	
	2012-07-30
	use dateutil.parser and calendar.timegm to convert no epoch times to the current locales time
		replaces function I wrote that only worked for UTC-4 to UTC+5 to Eastern time.
	remove last of old code that is no longer used
	
	2012-08-01
	remove last of leftover image setting variables
	
	2012-08-03
	add empty list to output when not using NOAA data to preserve the position of other data in the list
	
	2012-08-04
	send all the intellicast data to the lua script instead of just the snow chance data.

#################################################
v5.2.4 - 2012-07-17 - start of lua conversion
	setup VCLOUDS_STRING images to be drawn by lua script
	setup radar and sattelite images to be drawn by lua script
	
v5.2.3
	setup logging so conky can dispaly something if the script fails to update (due to script crash or lockup)
	check if WU API server sends html error document (encountered "High Load" error document crashed script because its is not json format)
v5.2.2
	removed textwrap import, I am not seeing the 80 character wrapping in output files without it anymore
v5.2.1
	Try to catch gifsicle errors indicating something is wrong with the gif retreived from Weather Underground (Did Not Work Error Message Not Caught)
		My not have been catching stderr added stderr=PIPE
		Yes now it works
	Fix For Weather Stations That Report No Value (To Weather Underground) Instead of Zero For Days Without Rain (api gives negative values instead of 0 or N/A)
	Fix Heat Index Temperature Horizontal Alignment
v5.2
	More Connection Error Handling Improvements - Looks Good So Far
		try around opening of data pulled from internet
		increase default timeout, increase timeout for json and image files

######################################
Requires
Font
	DejaVu Sans Mono - recommended may work with other monospace fonts

Weather Icons - Included
	VClouds Weather Icons - http://vclouds.deviantart.com/art/VClouds-Weather-Icons-179152045
	moonicons from conkyForecast - http://ubuntuforums.org/showthread.php?t=869328

luajson - http://www.eharning.us/wiki/luajson/
	json interpreter for lua
	package in Debian is "lua-json" (only in Wheezy and Sid)

python2.7
	Libraries used
	cPickle or pickle (cPickle is faster use it if possible)
	urllib2
	httplib
	socket
	json
	time
	datetime
	dateutil
	calendar
	subprocess
	
	packages in Debian are (only in Wheezy and Sid)
		python2.7 
		python-dateutil

gifsicle - http://www.lcdf.org/gifsicle/
	used to break the animated radar and satellite gifs into individual images for use in conky
	package in Debian is "gifsicle"

conky
	v1.9.0 or higher recommended.
	May work with v1.8.0.
	
	Do Not Use v1.8.1. Several things were broken in this release.
	
	lua support with cairo and imlib2 bindings required.
"""

try:
	import cPickle as pickle
	print "using cPickle"
except ImportError:
	import pickle
	print "using pickle"
#endtry
import urllib2
import httplib
import socket
import json
import time
import datetime
from dateutil import parser
from calendar import timegm
from subprocess import Popen, PIPE
### need gifsicle for animated radar ###

def tryconnect(turl, timer=4, retries=3):
	for tried in xrange(retries):
		try:
			return urllib2.urlopen(turl, None, timer)
		except (urllib2.URLError, socket.timeout, httplib.BadStatusLine), e:
			print "tryconnect Error: " + str(e)
		else:
			break
		#endtry
	#endfor
	return None
#enddef

def openURL(request_url, timer=30, retries=3):
	urlopener = tryconnect(request_url, timer, retries)
	if urlopener:
		try:
			return urlopener
		except (httplib.IncompleteRead, httplib.BadStatusLine, socket.timeout), e:
			print "openURL Error: " + str(e)
			return None
		#endtry
	else:
		return None
	#endif
#enddef

def conkyWeather():
	global logDirectory, pullAtempted, use_NOAA
	##### user defined variables #####
	#### System Variables ####
	installLocation = "Your Install location here" + "/conkyWeather_arclance_v6.0_release_2012-08-13" # location of the conkyWeather_arclance_v6.0.py file
	pollInterval = float(900) # time between data pulls from the API (in seconds) as a float
	logDirectory = installLocation + "/log" # directory to put working and log files in
	#### System Variables ####
	
	#### Wunderground API Variables	####
	APIKEY = "Your API Key Here" # the user specific API access key
	location = "Your Location Setting Here" # string specifying location to pull weather data for
	forecastSource = 0 # 1 = Weather Underground BestForecast | 0 = other source (National Weather Service in United States)
	usePersonalStations = 1 # use personal weather stations for condtions | 1 = true | 0 = false
	#### End Wunderground API Variables ####
	
	#### Wunderground API Radar ####
	radarWU_API_FEATURE = "animatedradar"
	radarWU_API_FORMAT = "gif"
	radarWU_API_LAT = "Lat" # your latitude here
	radarWU_API_LON = "Lon" # your longitude here
	radarWU_API_RADIUS = 150
	radarWU_API_SIZE = 500
	radarWU_API_NUMFRAMES = 15
	radarWU_API_TIMEPOSY = radarWU_API_SIZE - 5
	radarWU_API_PARAMS = "centerlat=" + str(radarWU_API_LAT) + "&centerlon=" + str(radarWU_API_LON) + "&radunits=km&radius=" + str(radarWU_API_RADIUS) + "&width=" + str(radarWU_API_SIZE) + "&height=" + str(radarWU_API_SIZE) + "&newmaps=1&rainsnow=1&reproj.automerc=1&num=" + str(radarWU_API_NUMFRAMES) + "&delay=75&timelabel=1&timelabel.x=0&timelabel.y=" + str(radarWU_API_TIMEPOSY)
	radarWU_API_URL = "http://api.wunderground.com/api/" + APIKEY + "/" + radarWU_API_FEATURE + "/image." + radarWU_API_FORMAT + "?" + radarWU_API_PARAMS
	#print radarWU_API_URL
	##### End Wunderground API Radar ####
	
	#### Wunderground Website Satellite ####
	satelliteWU_Animated_LAT = radarWU_API_LAT # your latitude here
	satelliteWU_Animated_LON = radarWU_API_LON # your longitude here
	satelliteWU_Animated_RADIUS = 150
	satelliteWU_Animated_SIZE = 500
	satelliteWU_Animated_NUMFRAMES = 15
	satelliteWU_Animated_TIMEPOSY = satelliteWU_Animated_SIZE - 5
	satelliteWU_Animated_TYPE = "sat_ir4_thumb" # type of satellite image to request
	satelliteWU_Animated_GTT = 107 # ground temperature threshhold. Level to cutoff and leave transparent. 107 is a reasonable value to show clouds but hide land.
	satelliteWUAnimated_URL = "http://wublast.wunderground.com/cgi-bin/WUBLAST?lat=" + str(satelliteWU_Animated_LAT) + "&lon=" + str(satelliteWU_Animated_LON) + "&radunits=km&radius=" + str(satelliteWU_Animated_RADIUS) + "&width=" + str(satelliteWU_Animated_SIZE) + "&height=" + str(satelliteWU_Animated_SIZE) + "&key=" + satelliteWU_Animated_TYPE + "&gtt=" + str(satelliteWU_Animated_GTT) + "&extension=gif&proj=me&num=" + str(satelliteWU_Animated_NUMFRAMES) + "&delay=75&timelabel=1&timelabel.x=0&timelabel.y=" + str(satelliteWU_Animated_TIMEPOSY) + "&basemap=1&borders=1&theme=WUNIDS&api_key=" + APIKEY #&rand=1330554982"
	#print(satelliteWUAnimated_URL)
	#### End Wunderground Website Satellite ####
	
	#### Intellicast variables ####
	intellicast_url = "Intellicast URL" # extended forcast url for intellicast.com
	#### End Intellicast Variables ####
	
	#### NOAA Variables ####
	use_NOAA = False # Get NOAA Floodstage Data | False = No , True = Yes
	NOAA_FloodstageURL = "NOAA Floodstage Graph URL" # River Level Floodstage Graph from NOAA
	NOAA_FloodstageTable = "NOAA Floodstage Table URL" # River Level Tabluated Data
	#### End NOAA Variables ####
	##### end user defined variables #####
	pullAtempted = False # stores if the first pull attempt has been made or not
	firstLoop = True # stroes if this is the first time the loop has run
	pullfailure = False # set pullfailure to false (no pulls made yet)
	while True: # run script until the process is killed
		if pullAtempted == False and firstLoop == True: # check if no pulls have been made and this is the first pass through the loop (only need to load backup lastPull time once)
			#lastPull = 0
			print "no pull yet"
			lastPullBackup = open(logDirectory + "/lastPullBackup.txt", "r") # file for keeping local copy of weather information incase of program/internet errors and to keep track of lastPull if the program is stopped
			lastPull = pickle.load(lastPullBackup) # read the last time an update was pulled from the api from the localBackup file (used to prevent exessive updates)
			lastPullBackup.close()
			errorCountLog = open((logDirectory + "/wuRadarPullErrorCountLog.txt"), 'r')
			errorCount = pickle.load(errorCountLog)
			errorCountLog.close()
			firstLoop = False # set firstLoop to false
		#endif
		currentTime = time.time() # the current time
		timeDifference = currentTime - lastPull
		print "\ntimeDifference = " + str(timeDifference)
		if timeDifference >= pollInterval: # if it has been longer than pollInterval since the last update, update the data
			print "pulling data"
			backupUsed = False
			##### WU DATA #####
			#### Wunderground API Variables ####
			infoString = "geolookup/conditions/forecast7day/hourly/astronomy/alerts" # specifies what data is pulled from the Wunderground API
			infoString = infoString + time.strftime("/history_%Y%m%d", time.localtime(time.time()))
			print infoString
			WU_API_URL = ('http://api.wunderground.com/api/' + APIKEY + "/" + infoString + "/bestfct:" + str(forecastSource) + "/pws:" + str(usePersonalStations) + "/q/" + location + ".json") # make the url request string
			#### End Wunderground API Variables ####
			print "Pulling -- WU Data -- Location = " + location
			weatherData = openURL(WU_API_URL, 100, 1) # read WU Weather Data
			if weatherData:
				try:
					####
					json_string = weatherData.read()
					json_split = json_string.split("\n")
					if json_split[0].strip() == "<html>": # not a json file (server sent error document)
						pullfailure = True
						if json_split[1].strip() == "<head><title>High Load</title>":
							print "WU API ERROR -- HIGH LOAD"
						else:
							print "WU API ERROR -- UNKNOWN ERROR"
						#endif
						wuapiErrorLog = open(logDirectory + "/wuapiErrorLog.txt", "w")
						wuapiErrorLog.write(json_string)
						wuapiErrorLog.close()
					else:
						localBackup = open(logDirectory + "/wuAPI_json_current_pickled.txt", "w") # file for keeping local copy of weather information incase of program/internet errors and to keep track of lastPull if the program is stopped
						pickle.dump(json_string, localBackup)
						localBackup.close()
						localBackupReadable = open(logDirectory + "/wuAPI_json_current_log.txt", "w") # readable json for determination of possible calls
						localBackupReadable.write(json_string)
						localBackupReadable.close()
						pullfailure = False
					#endif
					weatherData.close()
					####
				except (httplib.IncompleteRead, socket.timeout), e:
					print "json_string Read Error"
					print str(e)
					pullfailure = True
				#endtry
			else:
				pullfailure = True
			#endif
			##### END WU DATA #####
			##### Intellicast Data #####
			print "Pulling -- Intellicast Data"
			web_Data = openURL(intellicast_url) # read Intellicast Weather Data
			if web_Data:
				try:
					####
					webData = web_Data.read()				
					backupCopy2 = open(logDirectory + "/IntellicastForecast_LocalCopy.txt", "w") # load the backup file for writeing
					pickle.dump(webData, backupCopy2)
					intellicastData = intellicastForecast(webData)
					backupCopy2.close()
					web_Data.close()
					####
					pullfailure2 = False
				except (httplib.IncompleteRead, socket.timeout), e:
					print "intellicast Read Error"
					print str(e)
					pullfailure2 = True
				#endtry
			else:
				pullfailure2 = True
			#endif
			##### END Intellicast Data #####
			##### Radar/Satellite Data #####
			print "Pulling -- WU Radar"
			##### Radar Image From WU API #####
			radar_Data = openURL(radarWU_API_URL, 100, 1)
			if radar_Data:
				try:
					radarImage = radar_Data.read()
					radarFile = open((logDirectory + "/Images/radarImageA.gif"), 'wb')
					radarFile.write(radarImage)
					radarFile.close()
					(stdout, stderr) = Popen(["gifsicle", "--unoptimize", "--explode", (logDirectory + "/Images/radarImageA.gif"), "--output", (logDirectory + "/Images/radarImage_temp.gif")], stdout=PIPE, stderr=PIPE).communicate()
					#print str(stdout)
					radar_Data.close()
					#print "stdout = " + str(stdout)
					print "stderr = " + str(stderr).strip()
					if stderr != "":
						print "gifscile error"
						errorCount = errorCount + 1
						errorCountLog = open((logDirectory + "/wuRadarPullErrorCountLog.txt"), 'w')
						pickle.dump(errorCount, errorCountLog)
						errorCountLog.close()
						radarErrorOutput = ("Error Number - " + str(errorCount) + "\nError Time - " + time.strftime('%Y-%m-%d | %I:%M:%S %p %Z' ,time.localtime(float(time.time()))) +"\nError Document\n" + str(radarImage) + "\n")
						radarErrorFile = open((logDirectory + "/wuRadarPullErrorLog.txt"), 'a')
						radarErrorFile.write(radarErrorOutput)
						radarErrorFile.close()
						pullfailure3 = True
						backupUsed = True
					else:
						pullfailure3 = False
					#endif
					#pullfailure3 = False
				except (httplib.IncompleteRead, socket.timeout), e:
					print "radarImage Read Error"
					print str(e)
					pullfailure3 = True
					backupUsed = True
				#endtry
			else:
				pullfailure3 = True
				backupUsed = True
			#endif
			print "Pulling -- WU Satellite"
			satellite_Data = openURL(satelliteWUAnimated_URL, 100, 1)
			if satellite_Data:
				try:
					satelliteImage = satellite_Data.read()
					satelliteFile = open((logDirectory + "/Images/satelliteImageA.gif"), 'wb')
					satelliteFile.write(satelliteImage)
					satelliteFile.close()
					satellite_Data.close()
					(stdout, stderr) = Popen(["gifsicle", "--unoptimize", "--explode", (logDirectory + "/Images/satelliteImageA.gif"), "--output", (logDirectory + "/Images/satelliteImage_temp.gif")], stdout=PIPE, stderr=PIPE).communicate()
					#print "stdout = " + str(stdout)
					print "stderr = " + str(stderr).strip()
					if stderr != "":
						print "gifscile error"
						pullfailureSAT = True
						backupUsed = True
					else:
						pullfailureSAT = False
					#endif
					#print "\n" + str(stdout) + "\n" + str(stderr) + "\n"
					pullfailureSAT = False
				except (httplib.IncompleteRead, socket.timeout), e:
					print "satelliteImage Read Error"
					print str(e)
					pullfailureSAT = True
					backupUsed = True
				#endtry
			else:
				pullfailureSAT = True
				backupUsed = True
			#endif
			##### END Radar/Satellite Data #####
			##### NOAA Floodstage Data #####
			if use_NOAA == True:
				print "Pulling -- NOAA Floodstage Image"
				riverData = openURL(NOAA_FloodstageURL) # read current River Floodstage Graph
				if riverData:
					try:
						riverLevelImage = riverData.read()
						riverFile = open((logDirectory + "/Images/riverLevelImage.png"), 'wb')
						riverFile.write(riverLevelImage)
						riverFile.close()
						riverData.close()
					except (httplib.IncompleteRead, socket.timeout), e:
						print "riverLevelImage Read Error"
						print str(e)
					#endtry
				else:
					pass
				#endif
				print "Pulling -- NOAA Floodstage Table"
				riverTable = openURL(NOAA_FloodstageTable, 10, 4) # read current River Floodstage Level Info
				if riverTable:
					try:
						riverLevelTable = riverTable.read()
						riverTable.close()
						pullfailure4 = False
					except (httplib.IncompleteRead, socket.timeout), e:
						print "riverLevelTable Read Error"
						print str(e)
						pullfailure4 = True
					#endtry
				else:
					pullfailure4 = True
				#endif
			#endif
			##### END NOAA Floodstage Data #####
			######
			if pullfailure == True:
				#display results with warning about using backup data
				backupCopy = open(logDirectory + "/wuAPI_json_current_pickled.txt", "r") # load the backup file for reading
				weatherData = pickle.load(backupCopy) # read the backup data
				backupCopy.close()
				backupUsed = True
			else:
				lastPull = time.time() # update last pull to the current time
				lastPullBackup = open(logDirectory + "/lastPullBackup.txt", "w") # open lastPullBackup.txt for writeing
				pickle.dump(lastPull, lastPullBackup) # write the new last poll time to the log file
				lastPullBackup.close()
			#endif
			#######
			if pullfailure2 == True:
				## Intelicast
				# read from localBackup
				backupCopy2 = open(logDirectory + "/IntellicastForecast_LocalCopy.txt", "r") # load the backup file for reading
				webData = pickle.load(backupCopy2) # read the backup data
				intellicastData = intellicastForecast(webData)
				backupCopy2.close()
				backupUsed = True
			else:
				pass
			#endif
			#######
			if use_NOAA == True:
				if pullfailure4 == True:
					riverLevelBackup = open(logDirectory + "/NOAA_RiverLevelTable_LocalCopy.txt", "r")
					riverLevelTable = pickle.load(riverLevelBackup)
					riverLevelBackup.close()
					backupUsed = True
				else:
					riverLevelBackup = open(logDirectory + "/NOAA_RiverLevelTable_LocalCopy.txt", "w")
					pickle.dump(riverLevelTable, riverLevelBackup)
					riverLevelBackup.close()
				#endif
			else:
				riverLevelTable = ""
				pullfailure4 = False
			#endif
			#######
			print "Just Pulled -- Makeing Output"
			try:
				outputString = makeOutput(json_string, intellicastData, lastPull, riverLevelTable, backupUsed, pullfailure, pullfailure2, pullfailure3, pullfailure4, pullfailureSAT)
				#print outputString
			except (httplib.IncompleteRead, socket.timeout), e:
				print "Makeing Output Error"
				print str(e)
			else:
				weatherOutput = open(logDirectory + "/conkyWeather_arclance_lua_log.txt", "w") # load output file for writeing
				weatherOutput.write(outputString) # write output to text file for parsing by conky
				weatherOutput.close()
				print "Output Written"
				#break
			#endtry
			pullAtempted = True # set pullAtempted to true
		elif pullAtempted == False:
			print "using backup"
			#read from localBackup
			#display results with warning about using backup data
			backupCopy = open(logDirectory + "/wuAPI_json_current_pickled.txt", "r") # load the backup file for reading
			json_string = pickle.load(backupCopy) # read the backup data
			backupCopy.close()
			## Intelicast
			# read from localBackup
			backupCopy2 = open(logDirectory + "/IntellicastForecast_LocalCopy.txt", "r") # load the backup file for reading
			webData = pickle.load(backupCopy2) # read the backup data
			intellicastData = intellicastForecast(webData)
			backupCopy2.close()
			###
			if use_NOAA == True:
				riverLevelBackup = open(logDirectory + "/NOAA_RiverLevelTable_LocalCopy.txt", "r")
				riverLevelTable = pickle.load(riverLevelBackup)
				riverLevelBackup.close()
			else:
				riverLevelTable = ""
			#endif		
			# make output
			outputString = makeOutput(json_string, intellicastData, lastPull, riverLevelTable, True)
			weatherOutput = open(logDirectory + "/conkyWeather_arclance_lua_log.txt", "w") # load output file for writeing
			weatherOutput.write(outputString) # write output to text file for parsing by conky
			weatherOutput.close()					
			#print outputString
			print "Output Written"
		#endif
		time.sleep(5) # sleep for 5 seconds due to time measurement errors (just being safe)
		try: # catch errors returned by time.sleep() for debugging
			if pullfailure == True: # check if the pull failed
				nextUpdateSeconds = float(time.time() + (5*60))
				nextUpdateTime_Log = open (logDirectory + "/nextUpdateTime_Pickled.txt", "w")
				pickle.dump(nextUpdateSeconds, nextUpdateTime_Log)
				nextUpdateTime_Log.close()
				#nextUpdate = time.strftime('%Y-%m-%d | %I:%M:%S %p %Z' ,time.localtime(float(time.time() + (5*60)))) # convert epoch to localtime of lastPull
				nextUpdate = time.strftime('%Y-%m-%d | %I:%M:%S %p %Z', time.localtime(nextUpdateSeconds)) # convert epoch to localtime of lastPull
				print "Next Update At: " + nextUpdate
				time.sleep(5*60) # wait five minutes until next check if Wunderground Weather Data pull failed
			else: # calculate amount of time to wait until next update
				currentTime = time.time() # the current time
				timeDifference = float(currentTime) - float(lastPull)
				print "timeDifference = " + str(timeDifference)
				if timeDifference < pollInterval:
					print "waiting for " + str(pollInterval - timeDifference) + " s"
					nextUpdateSeconds = float(time.time() + (pollInterval - timeDifference)) # for logging
					nextUpdateTime_Log = open (logDirectory + "/nextUpdateTime_Pickled.txt", "w")
					pickle.dump(nextUpdateSeconds, nextUpdateTime_Log)
					nextUpdateTime_Log.close()
					#nextUpdate = time.strftime('%Y-%m-%d | %I:%M:%S %p %Z' ,time.localtime(float(time.time() + (pollInterval - timeDifference)))) # convert epoch to localtime of lastPull
					nextUpdate = time.strftime('%Y-%m-%d | %I:%M:%S %p %Z', time.localtime(nextUpdateSeconds)) # convert epoch to localtime of lastPull
					print "Next Update At: " + nextUpdate
					time.sleep(pollInterval - timeDifference) # wait until next schedualed update interval
				#endif
			#endif
		except (IOError), e:
			print "Invalid time.sleep() Argument Error -- " + str(e)
			print "currentTime = " + str(float(currentTime))
			print "lastPull = " + str(float(lastPull))
			print "timeDifference = " + str(timeDifference)
			print "pollInterval = " + str(pollInterval)
			print "pollInterval - timeDifference = " + str(pollInterval - timeDifference)
			nextUpdateSeconds = float(time.time() + (10*60))
			nextUpdateTime_Log = open (logDirectory + "/nextUpdateTime_Pickled.txt", "w")
			pickle.dump(nextUpdateSeconds, nextUpdateTime_Log)
			nextUpdateTime_Log.close()
			nextUpdate = time.strftime('%Y-%m-%d | %I:%M:%S %p %Z', time.localtime(nextUpdateSeconds)) # convert epoch to localtime of lastPull
			print "Next Update At: " + nextUpdate
			time.sleep(10*60) # wait 10 minuets until next check if time.sleep(pollInterval - timeDifference) returns an error
		#endtry	
	#endwhile
#enddef

def makeOutput(json_string, intellicastData, lastPull, riverLevelTable, backupUsed = False, pullFailure = False, pullFailure2 = False, pullFailure3 = False, pullFailure4 = False, pullFailureSAT = False): # makes the output string
	print 'backupUsed: ' + str(backupUsed)
	print 'pullFailure: ' + str(pullFailure)
	print 'pullFailure2: ' + str(pullFailure2)
	print 'pullFailure3: ' + str(pullFailure3)
	print 'pullFailure4: ' + str(pullFailure4)
	print 'pullFailureSAT: ' + str(pullFailureSAT)
	############
	luaOutput = []
	##### River Level (Flood Monitoring) #####
	if use_NOAA == True:
		riverLevelTableSplit = riverLevelTable.split('\n')
		dateSplit = [0,0,0]
		dateSplit[0] = (time.strftime('%Y' ,time.localtime(float(lastPull))))
		dateStart = riverLevelTableSplit[36].find("<td nowrap>") + len("<td nowrap>")
		dateEnd = riverLevelTableSplit[36].find("</td>")
		dateString = riverLevelTableSplit[36][dateStart:dateEnd]
		dateSplit[1] = (dateString.split(' ')[0].split('/')[0])
		dateSplit[2] = (dateString.split(' ')[0].split('/')[1])
		timeUTCSplit = dateString.split(' ')[1].split(':')
		[localTime, localDate, AMorPM] = convertTime_to_Eastern(timeUTCSplit, dateSplit) #
		currentRiverReading_Date = "-".join(localDate)
		currentRiverReading_Time12hr = ":".join(localTime) + " " + AMorPM # make local time string
		currentRiverReading_Level = riverLevelTableSplit[37][21:27].strip('ft</') + ' ft'
		print currentRiverReading_Date + " " + str(currentRiverReading_Time12hr) + " " + str(currentRiverReading_Level)
		localTime.append(AMorPM)
		riverDataLua = [currentRiverReading_Level,localDate,localTime]
		luaOutput.append(riverDataLua)
	else:
		luaOutput.append([])
	#endif
	#### End River Level (Flood Monitoring) ####
	#### make final output string ####
	lastUpdate = time.strftime(' | %I:%M:%S %p %Z' ,time.localtime(float(lastPull))) # convert epoch to localtime of lastPull
	print "lastUpdate = " + lastUpdate
	networkStateLua = [[0,[0,0,0,0,0]],[0,[0,0,0,0,0]]] ## [[backupUsed,[W,I,R,S,N]],[connState,[W,I,R,S,N]]]
	### Make backupString ###
	if backupUsed == True:
		networkStateLua[0][0] = 1
		if pullAtempted == False:
			networkStateLua[0][1] = [1,1,1,1,1]
		else:
			if pullFailure == True:
				networkStateLua[0][1][0] = 1
			#endif
			if pullFailure2 == True:
				networkStateLua[0][1][1] = 1
			#endif
			if pullFailure3 == True:
				networkStateLua[0][1][2] = 1
			#endif
			if pullFailureSAT == True:
				networkStateLua[0][1][3] = 1
			#endif
			if pullFailure4 == True:
				networkStateLua[0][1][4] = 1
			#endif
		#endif
	#endif
	### Make connectionString ###
	if pullAtempted == True and ((pullFailure == True) or (pullFailure2 == True) or (pullFailure3 == True) or (pullFailure4 == True) or (pullFailureSAT == True)):
		networkStateLua[1][0] = 1
		if pullFailure == True:
			networkStateLua[1][1][0] = 1
		#endif
		if pullFailure2 == True:
			networkStateLua[1][1][1] = 1
		#endif
		if pullFailure3 == True:
			networkStateLua[1][1][2] = 1
		#endif
		if pullFailureSAT == True:
			networkStateLua[1][1][3] = 1
		#endif
		if pullFailure4 == True:
			networkStateLua[1][1][4] = 1
		#endif
	#endif
	luaOutput.append(networkStateLua)
	####
	iCast_Lua = []
	for i in xrange(len(intellicastData)):
		iCast_Lua.append([intellicastData[i].getConditions(), intellicastData[i].getHiTemp(), intellicastData[i].getLowTemp(), intellicastData[i].getSunrise(), intellicastData[i].getSunset(), intellicastData[i].getMoonrise(), intellicastData[i].getMoonset(), intellicastData[i].getUVLevel(), intellicastData[i].getHumidity(), intellicastData[i].getPrecipChance(), intellicastData[i].getSnowChance(), intellicastData[i].getCloudCoverage(), intellicastData[i].getMoonPhase(), intellicastData[i].getWindSpeed(), intellicastData[i].getWindDirection()])
	#endfor
	luaOutput.append(iCast_Lua)
	luaOutput = str(luaOutput).replace("[","{").replace("]","}").replace("'{","{").replace("}'","}").replace("'\"","'").replace("\"'","'")
	return "return " + luaOutput
#enddef

###### Intellicast Section ######
def intellicastForecast(web_data):
	web_dataSplit = web_data.split("\n")
	daynum = -1
	for i in xrange(len(web_dataSplit)):
		currentLine = web_dataSplit[i].strip()
		checkConditions = currentLine.find('<td style="padding-top:15px;">')
		checkHiTemp = currentLine.find('<td><span class="Hi">')
		checkRise = currentLine.find("Rise:</strong>")
		checkSet = currentLine.find("Set:</strong>")
		checkUV = currentLine.find('<strong>UV Index:</strong>')
		checkHumidity = currentLine.find('Relative Humidity:</strong>')
		checkPrecipChance = currentLine.find('Precipitation:</strong>')
		checkSnowChance = currentLine.find('Snow Probability:</strong>')
		checkCloudCoverage = currentLine.find('Cloud Coverage:</strong>')
		checkMoonPhase = currentLine.find('Moon Phase:</strong>')
		checkWindSpeed = currentLine.find('Wind Speed:</strong>')
		checkWindDirection = currentLine.find('Wind Direction:</strong>')	
		if currentLine == '<table id="extended">':
			daynum = daynum + 1
			makeDataObj = "day" + str(daynum) + " = forecastData()"
			exec makeDataObj
			makeGlobal = "global day" + str(daynum)
			exec makeGlobal
			riseType = 0
			setType = 0
		elif checkConditions != (-1):
			checkConditionsStart = currentLine.find('<br />')
			stringStart = int(checkConditionsStart + len('<br />'))
			stringEnd = currentLine.find('</td>')
			conditions = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setConditions(conditions)"
			exec setDataObj
		elif checkHiTemp != (-1):
			stringStart = int(checkHiTemp + len('<td><span class="Hi">'))
			stringEnd = currentLine.find('</span><br /><span class="Lo">')
			hightemp = currentLine[stringStart:stringEnd].strip('&deg')
			setDataObj = "day" + str(daynum) + ".setHiTemp(hightemp)"
			exec setDataObj
			findLowTemp = currentLine.find('<span class="Lo">')
			stringStart = int(findLowTemp + len('<span class="Lo">'))
			stringEnd = currentLine.find('</span></td>')
			lowtemp = currentLine[stringStart:stringEnd].strip('&deg;')
			setDataObj = "day" + str(daynum) + ".setLowTemp(lowtemp)"
			exec setDataObj
		elif checkRise != (-1):
			stringStart = int(checkRise + len("Rise:</strong>"))
			stringEnd = currentLine.find('</td>')
			if riseType == 0:
				sunRise = currentLine[stringStart:stringEnd].strip()
				setDataObj = "day" + str(daynum) + ".setSunrise(sunRise)"
				exec setDataObj
				riseType += 1
			elif riseType == 1:
				moonRise = currentLine[stringStart:stringEnd].strip()
				setDataObj = "day" + str(daynum) + ".setMoonrise(moonRise)"
				exec setDataObj
			#endif
		elif checkSet != (-1):
			stringStart = int(checkSet + len("Rise:</strong>"))
			stringEnd = currentLine.find('</td>')
			if setType == 0:
				sunSet = currentLine[stringStart:stringEnd].strip()
				setDataObj = "day" + str(daynum) + ".setSunset(sunSet)"
				exec setDataObj
				setType += 1
			elif setType == 1:
				moonSet = currentLine[stringStart:stringEnd].strip()
				setDataObj = "day" + str(daynum) + ".setMoonset(moonSet)"
				exec setDataObj			
			#endif
		elif checkUV != (-1):
			stringStart = int(checkUV + len('<strong>UV Index:</strong>'))
			stringEnd = currentLine.find('<br />')
			UVLevel = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setUVLevel(UVLevel)"
			exec setDataObj
		elif checkHumidity != (-1):
			stringStart = int(checkHumidity + len('Relative Humidity:</strong>'))
			stringEnd = currentLine.find('<br />')
			humidity = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setHumidity(humidity)"
			exec setDataObj				
		elif checkPrecipChance != (-1):
			stringStart = int(checkPrecipChance + len('Precipitation:</strong>'))
			stringEnd = currentLine.find('<br />')
			precipChance = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setPrecipChance(precipChance)"
			exec setDataObj					
		elif checkSnowChance != (-1):
			snowChance = str(web_dataSplit[i+1]).strip().strip(" <br/>")
			setDataObj = "day" + str(daynum) + ".setSnowChance(snowChance)"
			exec setDataObj		
		elif checkCloudCoverage != (-1):
			stringStart = int(checkCloudCoverage + len('Cloud Coverage:</strong>'))
			stringEnd = currentLine.find('<br />')
			cloudCoverage = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setCloudCoverage(cloudCoverage)"
			exec setDataObj					
		elif checkMoonPhase != (-1):
			stringStart = int(checkMoonPhase + len('Moon Phase:</strong>'))
			stringEnd = currentLine.find('<br />')
			moonPhase = currentLine[stringStart:stringEnd].strip()
			setDataObj = "day" + str(daynum) + ".setMoonPhase(moonPhase)"
			exec setDataObj		
		elif checkWindSpeed != (-1):
			stringStart = int(checkWindSpeed + len('Wind Speed:</strong>'))
			stringEnd = currentLine.find('(')
			windSpeed = currentLine[stringStart:stringEnd].strip(' Mph') # in mph
			setDataObj = "day" + str(daynum) + ".setWindSpeed(windSpeed)"
			exec setDataObj
		elif checkWindDirection != (-1):
			stringStart = int(checkWindDirection + len('Wind Direction:</strong>'))
			stringEnd = currentLine.find('(')
			windDirectionDeg = currentLine[stringStart:stringEnd].strip(' &deg;') # in degrees
			stringStart = stringEnd + 1
			stringEnd = currentLine.find(')')
			windDirectionCard = currentLine[stringStart:stringEnd].strip() # in degrees
			setDataObj = "day" + str(daynum) + ".setWindDirection(windDirectionDeg, windDirectionCard)"
			exec setDataObj											
		#endif		
	#endfor
	return [day0,day1,day2,day3,day4,day5,day6,day7,day8,day9]
#enddef	
	
class forecastData():
	
	def __init__(self):
		self._Conditions = "N/A"
		self._highTemp = "N/A"
		self._lowTemp = "N/A"
		self._sunRiseTime = "N/A"
		self._sunSetTime = "N/A"
		self._moonRiseTime = "N/A"
		self._moonSetTime = "N/A"
		self._UVLevel =  "N/A"
		self._Humidity =  "N/A"
		self._precipChance =  "N/A"
		self._snowChance =  "N/A"
		self._cloudCoverage =  "N/A"
		self._moonPhase =  "N/A"
		self._windSpeed =  "N/A"
		self._windDirDeg = "N/A"
		self._windDirCard = "N/A"
	#enddef
	
	def setConditions(self, conditions):
		self._Conditions = conditions
	#enddef
	
	def getConditions(self):
		return self._Conditions
	#enddef		
	
	def setHiTemp(self, highTemp):
		self._highTemp = highTemp
	#enddef
	
	def getHiTemp(self):
		return self._highTemp
	#enddef	
	
	def setLowTemp(self, lowTemp):
		self._lowTemp = lowTemp
	#enddef
	
	def getLowTemp(self):
		return self._lowTemp
	#enddef
	
	def setSunrise(self, sunRiseTime):
		self._sunRiseTime = sunRiseTime
	#enddef
	
	def getSunrise(self):
		return self._sunRiseTime
	#enddef
	
	def setSunset(self, sunSetTime):
		self._sunSetTime = sunSetTime
	#enddef
	
	def	getSunset(self):
		return self._sunSetTime
	#enddef
	
	def setMoonrise(self, moonRiseTime):
		self._moonRiseTime = moonRiseTime
	#enddef
	
	def getMoonrise(self):
		return self._moonRiseTime
	#enddef	
	
	def setMoonset(self, moonSetTime):
		self._moonSetTime = moonSetTime
	#enddef
	
	def getMoonset(self):
		return self._moonSetTime
	#enddef
	
	def setUVLevel(self, UVLevel):
		self._UVLevel = UVLevel
	#enddef
	
	def getUVLevel(self):
		return self._UVLevel
	#enddef
	
	def setHumidity(self, Humidity):
		self._Humidity = Humidity
	#enddef	
	
	def getHumidity(self):
		return self._Humidity
	#enddef
	
	def setPrecipChance(self, precipChance):
		self._precipChance = precipChance
	#enddef
	
	def getPrecipChance(self):
		return self._precipChance
	#enddef
	
	def setSnowChance(self, snowChance):
		self._snowChance = snowChance
	#enddef
	
	def getSnowChance(self):
		return self._snowChance
	#enddef
	
	def setCloudCoverage(self, cloudCoverage):
		self._cloudCoverage = cloudCoverage
	#enddef
	
	def getCloudCoverage(self):
		return self._cloudCoverage
	#enddef
	
	def setMoonPhase(self, moonPhase):
		self._moonPhase = moonPhase
	#enddef
	
	def getMoonPhase(self):
		return self._moonPhase
	#enddef
	
	def setWindSpeed(self, windSpeed):
		self._windSpeed = windSpeed
	#enddef
	
	def getWindSpeed(self):
		return self._windSpeed
	#enddef
	
	def setWindDirection(self, windDirDeg, windDirCard):
		self._windDirDeg = windDirDeg
		self._windDirCard = windDirCard
	#enddef
	
	def getWindDirection(self):
		return [self._windDirDeg,self._windDirCard]
	#enddef
#enddef
#### End Intellicast Section ####

def convertTime_to_Eastern(timeUTCSplit, dateSplit):
	timeStruct = parser.parse(dateSplit[0] + "-" + dateSplit[1] + "-" + dateSplit[2] + " " + timeUTCSplit[0] + ":" + timeUTCSplit[1]).utctimetuple()
	timeEpoch = timegm(timeStruct)
	localStruct = time.localtime(timeEpoch)
	localTime = time.strftime("%I:%M",localStruct)
	localDate = time.strftime("%y-%m-%d",localStruct)
	AMorPM = time.strftime("%p",localStruct)
	return [localTime.split(":"), localDate.split("-"), AMorPM]
#enddef

conkyWeather()
